<?php
/*

Copyright  2001, 2002, 2003, 2004 Andres Baravalle

This file is part of phpGrabComics.

phpGrabComics is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

phpGrabComics is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with phpGrabComics; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


/*
Used convetion that functions starting with:
-p: print something
-r: return something
-m or no prefix: modify some class value or make something
-h: add header
*/

/**
 * comic grabbing class.
 *
 * @author Andres Baravalle <andres@baravalle.it>
 *
*/
class phpGrabComics
{
	// You should not need to modify anything else but comic_config.inc for normal purposes.
	// Any improvement is welcome (-:
	
	// url for image
	var $img_url = "";
	
	// name of image (it's used for deciding file name)
	var $img_name = "";
	
	var $address = "";
	
	// html code of retrieved page
	var $comic_html = "";
	
	// string to be matched for grabbing
	var $match_string = "";
	
	var $match_prefix = False;
	
	// unix name of comic (eg. "calvinandhobbes", used for directory names etc.).
	var $unix_name = "";
	
	// name of the comic
	var $name = "";
	
	// Suffix for today image.
	// image is saved in the temporary position and only if it is ok is then copied to the correct position
	var $img_today_suffix = "_today";
	
	var $referer = "";
	
	// default extension
	var $img_ext = ".gif";
	
	// valid mimes; very untidy way, but available in most php versions
	var $valid_mimes = Array(
		// default extension, magic number, type of magic number, mime type
		Array("gif","GIF8","string","image/gif"),
		Array("jpg","0xffd8","beshort","image/jpeg"),
		Array("png","\x89PNG","string","image/png")
	);
	
	// size in bytes of the minimum dimension for comics. if a comics is less then that, will be deleted.
	var $min_filesize = 3000;
	
	// if to report errors or not
	var $report = True;
	
	/**
	* @return phpGrabComics
	* @desc base constructor. defines name and unix name if not provided in the extended classes.
	*/
	function phpGrabComics()
	{
		if(empty($this->name))
		$this->name = strtoupper(substr(get_class($this),0,1)).substr(get_class($this),1);
		if(empty($this->unix_name))
		$this->unix_name = strtolower(get_class($this));
	}	
	
	/**
	* @return boolean
	* @desc tries to grab choosen comic
	*/
	function grab()
	{
		global $_GET;
		global $dir;
		
		// this is needed to fake some sites
		if(empty($this->referer))
		$this->referer=$this->address;
		
		// Now we are copying today's comic to local file system
		
		$tmp_return = False;
		
		/*
		there are some special cases
		1- if there is is already a *tmp file with current date you don't need to grab anything
		2- if there is not already a tmp file, you have to grab the page; if there is already a file with that name in the comics folder, you don't need to grab anything
		*/
		
		if($checkgrab=$this->checkGrab() and $grabhtml=$this->grabHtml() and $graburl=$this->grabUrl() and $getfilename=$this->getFileName())
		{
			$img_save_path = $GLOBALS["tmp_files"] . DIRECTORY_SEPARATOR . $this->unix_name . $this->img_today_suffix;
			
			$img_data = $this->read($this->img_url);
			
			if($this->rCheckMime(substr($img_data,0,4))!==False)
			{
				// we do not want the script to be aborted when we are accessing to the file system.
				ignore_user_abort(True);
				clearstatcache();
				
				if(is_writable($GLOBALS["tmp_files"]))
				{
					$file2=fopen($img_save_path,"wb");
					
					if ($file2)
					{
						if(flock($file2,LOCK_EX))
						{
							if(fwrite($file2, $img_data))
							{
								if(!isset($_GET["view"]) and !isset($_GET["show"]))
								printf(_IMG_SAVED_OK, $img_save_path);
								
								flock($file2, LOCK_UN);
								fclose($file2);
								
								if(filesize($img_save_path)<$this->min_filesize)
								{
									trigger_error(sprintf(_TOO_SMALL_IMAGE,$this->img_url),E_USER_WARNING );
									$error = sprintf(_TOO_SMALL_IMAGE,$this->img_url);
									unlink($img_save_path);
								}
								else
								{
									copy($img_save_path,$GLOBALS["comics"] . DIRECTORY_SEPARATOR . $this->unix_name . $this->img_today_suffix);
									
									// if the comic is copy it to the archive.
									if(in_array($this->unix_name,$GLOBALS["comics_updated"]))
									{
										if(!is_dir($GLOBALS["comics"] . DIRECTORY_SEPARATOR . $this->unix_name))
										{
											mkdir($GLOBALS["comics"] . DIRECTORY_SEPARATOR . $this->unix_name,0777);
										}
										elseif((bool)ini_get("safe_mode")===False and !is_writable($GLOBALS["comics"] . "/" . $this->unix_name))
										{
											@chmod($GLOBALS["comics"] . "/" . $this->unix_name,0777);
										}
										
										copy($img_save_path,$GLOBALS["comics"] . "/" . $this->unix_name . "/" . $this->img_name);
									}
									
									$tmp_return = True;
									
									printf(_IMG_SAVED_CORRECTLY, $this->name);
								}
							}
							else
							{
								trigger_error(sprintf(_SAVE_FILE,$this->img_url,$img_save_path),E_USER_WARNING );
								$error = sprintf(_SAVE_FILE,$this->img_url,$img_save_path);
							}
							
							// if exists, delete the temporary copy.
							if(is_file($img_save_path))
								unlink($img_save_path);
						}
						else
						{
							trigger_error(sprintf(_NO_LOCK,$img_save_path),E_USER_WARNING );
							$error = sprintf(_NO_LOCK,$img_save_path);
						}
					}
					else
					{
						trigger_error(sprintf(_OPEN_DEST_FILE,$img_save_path,$this->img_url),E_USER_WARNING );
						$error = sprintf(_OPEN_DEST_FILE,$img_save_path,$this->img_url);
					}
				}
				else
				{
					trigger_error(sprintf(_NO_WRITABLE,$GLOBALS["tmp_files"],$this->img_url),E_USER_WARNING );
					$error = sprintf(_NO_WRITABLE,$GLOBALS["tmp_files"],$this->img_url);
				}
			}
			else
			{
				$error = sprintf(_BROKEN_IMAGE,$img_save_path,$this->img_url);
			}
			
			ignore_user_abort(False);
		}
		else
		{
			if($checkgrab==False)
			{
				// only checkgrab returned false. so you don't need to grab, but there are no errors.
				if($_GET["save"]==1)
				printf("<p>Don't need to save.</p>");
				$tmp_return==False;
			}
			else if($grabhtml==False)
			{
				trigger_error(sprintf(_GRAB_HTML,$this->address),E_USER_WARNING);
				$error = sprintf(_GRAB_HTML,$this->address);
			}
			else if($graburl==False)
			{
				trigger_error(sprintf(_GRAB_URL,$this->address),E_USER_WARNING);
				$error = sprintf(_GRAB_URL,$this->address);
			}
			else if($getfilename==False)
			{
				trigger_error(sprintf(_FILENAME,$this->address),E_USER_WARNING);
				$error = sprintf(_FILENAME,$this->address);
			}
		}
		
		if($tmp_return==False and $GLOBALS["notify"]!==False and isset($error) and $this->report===True)
		mail($GLOBALS["notify"],"phpGrabComics error in " . $this->unix_name,strip_tags($error));
		
		if($tmp_return==False and isset($error) and $this->report===True)
		{
			if(!is_writable($dir."error_log.csv") and	(bool)ini_get("safe_mode")===False)
			{
				@chmod($GLOBALS["comics"] . "/" . $this->unix_name,0777);
			}
			
			if(is_writable($dir."error_log.csv"))
			{
				$file3=fopen($dir."error_log.csv","a");
				
				if ($file3)
				{
					if(flock($file3,LOCK_EX))
					{
						fwrite($file3, "\"".date("d/m/Y H:i:s",time()) . "\",\"".$this->unix_name."\",\"" .trim(strip_tags($error)) ."\"\n");
						flock($file3, LOCK_UN);
					}
					fclose($file3);
				}
			}
		}
		
		return $tmp_return;
	}
	
	
	/**
	* @return boolean
	* @desc this function may be overridden by the different classes. checks if the file has been already retrieved in the last hours or not
	*/
	function checkGrab()
	{
		global $_GET;
		
		$tmp_return = False;
		
		$time_slice = 4;
		$now = date("YmdH", time());
		$file = $GLOBALS["comics"] . "/" . $this->unix_name . $this->img_today_suffix;
		
		// checks if the files exists
		if(is_file($file))
		{
			clearstatcache();
			$file_date = @date("YmdH", filemtime($file));
			
			if($now>$file_date+$time_slice)
				$tmp_return = True;
		}
		else
			$tmp_return = True;
		
		return $tmp_return;
	}
	
	/**
	* @return False or resource
	* @param $url URL of resource to download
	* @desc wide-compatible download function
	*/
	function read($url)
	{
		// using real headers. some sites check for some strings to be present in the headers.
		$http_headers[] = Array("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0; .NET CLR 1.1.4322");
		$http_headers[] = Array("Accept","image/png,image/jpeg,image/gif;q=0.2,*/*;q=0.1");
		$http_headers[] = Array("Accept-Language","en-us,en;q=0.5");
		$http_headers[] = Array("Accept-Charset","ISO-8859-1,utf-8;q=0.7,*;q=0.7");
		$http_headers[] = Array("Referer",$this->referer);
		
		$req = &new HTTP_PhpGrabComics_Request($url);
		
		while (list($key,$val) = each($http_headers))
		{
			$req->addHeader($val[0],$val[1]);	
		}
			
		$req->sendRequest();
	
		$response = $req->getResponseBody();	
		
		return $response;
	}

	/**
	* @return void
	* @desc prints the archive of selected comics
	*/
	function pListComics()
	{
		global $dir;
		
		$image_list = Array();
		
		if ($handle = @opendir($dir . $GLOBALS["comics"]."/".$this->unix_name))
		{
			while (false !== ($file = readdir($handle)))
			{
				if ($file != "." && $file != "..")
				{
					$time = filemtime($dir . $GLOBALS["comics"] ."/" .$this->unix_name . "/" . $file);
					
					$time_list[] = $time;
					
					$image_list[] = Array($time, $file);
				}
			}
			closedir($handle);
			
			if(isset($image_list) and !empty($image_list))
			{
				array_multisort($image_list, SORT_DESC, $time_list);
				
				echo "<table>";
				
				$cols=6;
				$count=1;
				
				while(list($key,$val)=each($image_list))
				{
					if($count==1)
					echo "<tr>";
					
					echo "<td>";
					echo "<a href=\"" . $_SERVER["SCRIPT_NAME"] . "?comic_array[]=".$this->unix_name."&amp;show_img=" . $val[1] ."\">" . date("Y/m/d", $val[0]) ."</a>";
					echo "</td>";
					
					if($count==6)
					{
						echo "</tr>\n";
						$count = 1;
					}
					else
					$count++;
				}
				
				for(;$count<=$cols;$count++)
				{
					echo "<td>&nbsp;</td>";
					if($count==$cols)
					echo "</tr>\n";
				}
				
				echo "</table>";
				printf(_IMG_AVAILABLE, sizeof($image_list));
			}
			else
			{
				echo _NO_COMICS;
			}
		}
		else
		echo _NO_COMICS;
	}
	
	/**
	* @return string
	* @desc returns comic icon for this class. it is intended to be inheritated and used by a sub-class
	*/
	function rIcon($case="default")
	{
		global $_SERVER;
		global $dir;
		
		$tmp_return = "<a href=\"".$dir."index.php?comic_array[]=". $this->unix_name;
		
		if($case=="view")
			$tmp_return .= "&amp;view=1";
		elseif($case=="list_strips")
			$tmp_return .= "&amp;list_strips=1";
		
		$tmp_return .= "\">";
		
		$img_path = $GLOBALS["config"] . "/". $this->unix_name . ".png";
		
		if (!file_exists($img_path))
		{
			$img_path = $GLOBALS["config"] . "/". $GLOBALS["default_icon"];
		}
		
		$tmp_return .= "<img src=\"".$img_path."\"";
		$tmp_return .= " alt=\"" . $this->name ."\" width=\"32\" height=\"32\" ";
		
		if($case=="list_strips")
		$tmp_return .= "style=\"text-align: center; vertical-align: middle\" />";
		else		
		$tmp_return .= "style=\"float: left; vertical-align: middle;\" />";
		
		$tmp_return .= "</a>";
		
		return $tmp_return;
	}

	/**
	* @return void
	* @param $img path of image to be inserted in the page
	* @desc checks if image file exists. it so, it prints html code for displaying.
 	*/
	function pImg($img="")
	{
		global $dir;
		global $comic_list;
		
		if($this->unix_name=="random")
		{
			srand(time());
			$rand_key = array_rand($comic_list, 1);

			$img_path = $dir . $GLOBALS["show_comic"] ."/" . $comic_list[$rand_key]->unix_name;
			$test_path = $dir . $GLOBALS["comics"] . "/" .$comic_list[$rand_key]->unix_name . $comic_list[$rand_key]->img_today_suffix;
		}
		elseif($img!="")
		{
			$img_path = $GLOBALS["comics"] . "/" .  $img;
			$test_path = $img_path;
		}
		else
		{
			$img_path = $dir . $GLOBALS["show_comic"] ."/" . $this->unix_name;
			$test_path = $dir . $GLOBALS["comics"] . "/" . $this->unix_name . $this->img_today_suffix;
		}
		
		if(@fopen($test_path,"r"))
		{
			echo "<img src=\"$img_path\" alt=\"". $this->name;
			if($img!="")
			echo " - " . $img;
			echo "\" />";
			echo "<br />\n<br />\n";
		}
		else
		{
			echo "<img src=\"media/phpgrabcomics_error.png\" alt=\"". _NOT_AVAILABLE ."\" />";
		}
	}
	
	/**
	* @return void
	* @param $img path of image
	* @param $alt alt text
	* @desc prints html code for displaying image $img without checking if file exists. check must be done before calling this function.
 	*/
	function pImgRelative($img,$alt)
	{
		echo "<img src=\"$img\" alt=\"$alt\" />";
		echo "<br />\n<br />\n";
	}
	
	/**
	* @return unknown
	* @param string $img
	* @desc outputs a image. if $img is empy, outputs today's comic, else outputs $img.
	*/
	function makeImg($img="")
	{
		global $dir;
		global $comic_list;
		
		if($this->unix_name=="random")
		{
			srand(time());
			$rand_key = array_rand($comic_list, 1);

			$img = $dir . $GLOBALS["comics"] . "/" .$comic_list[$rand_key]->unix_name . $comic_list[$rand_key]->img_today_suffix;
		}
		elseif(empty($img))
		{
			$img = $dir . $GLOBALS["comics"] . "/" .$this->unix_name . $this->img_today_suffix;
		}
		
		if(is_file($img))
		{
			if($file = fopen($img,"rb"))
			{
				$contents = fread($file, filesize($img));
				$mime = $this->rCheckMime(substr($contents,0,4));
				if($mime!==False)
				{
					header("Content-type: " . $mime[3]);
				}
				echo $contents;			
				return True;
			}
			else
			{
				return False;
			}
		}
	}
	
	/**
	* @return void
	* @param $img
	* @desc makes a counter based on a image	
 	*/
	function makeJpegCounter($img)
	{
		global $_SERVER;		
		global $db_user;
		global $db_psw;
		global $db_host;
		global $db_name;
		global $db_prefix;
		
		if(mysql_connect($db_host,$db_user,$db_psw) and mysql_select_db($db_name))
		{
			if(isset($_SERVER["HTTP_REFERER"]))
			{
				$tmp_page = parse_url($_SERVER["HTTP_REFERER"]);
				if(strpos($tmp_page["host"],"www.")!==False)
				$tmp_page["host"]=str_replace("www.","",$tmp_page["host"]);
				$page = $tmp_page["host"] . $tmp_page["path"];
			}
			else
			$page = "test";
			
			$sql = "select * FROM ${db_prefix}_sites where page = '$page';";
			
			$result = mysql_query($sql);
			
			if(mysql_num_rows($result)>0)
			{
				$row = mysql_fetch_array($result, MYSQL_ASSOC);
				$string = $row["credits"] . " hits!";
			}
			else
			$string = " no hits )-:";
		}
		
		header ("Content-type: image/jpeg");
		$im = imagecreatefromjpeg($img);
		
		$x = imagesx($im) - 8*strlen($string);
		$y = imagesy($im) - 15;
		
		$color = imagecolorallocate($im, 0, 0, 0);
		
		imagestring($im,3,$x,$y,$string,$color);
		
		imagejpeg($im);
		imagedestroy($im);
	}
	
	
	/**
	* @return void
	* @desc prints html code for displaying image of active comic. 
	* can be redefined in different comics modules	
 	*/
	function pGrabUrl()
	{
		if(isset($this->img_url) and !empty($this->img_url))
		{
			$html="<img src=\"" . $this->img_url . "\" alt=\"" . $this->name ."\" />";
			
			echo $html;
		}
		else
		trigger_error(sprintf(_NO_IMG_URL,$this->name),E_USER_WARNING);
	}
	
	/**
	* @return boolean
	* @desc grabs html from the url of a comic 
	* can be redefined in different comics modules	
 	*/
	function grabHtml()
	{
		$data = $this->read($this->address);
		
		if($data!==False)
		{
			$this->comic_html = $data;
			return True;
		}
		else
		return False;
	}
	
	/**
	* @return boolean
	* @desc grabs an url from the html grabbed from the url of a comic
	* can be redefined in different comics modules
 	*/
	function grabUrl()
	{
		$tmp_return = False;
		
		if($this->match_string!="" and $this->comic_html!="")
		{
			if (preg_match($this->match_string, $this->comic_html, $matches))
			{
				if($this->match_prefix===False)
				{
					preg_match("/http:\/\/[^\/]*/",$this->address,$matches2);

					if(preg_match("/^\/.*/",$matches[1]))
					$this->match_prefix = $matches2[0];
					else
					$this->match_prefix = $matches2[0] . "/";
				}
				
				$this->img_url = $this->match_prefix . $matches[1];
				
				if(!isset($this->img_ext) and isset($matches[2]))
				{
					$this->img_ext=$matches[0];
				}
				
				$tmp_return = $this->img_url;
			}
		}
		
		return $tmp_return;
	}
	
	/**
	* @return boolean
	* @desc gets the name that the strip will have when saved in the archive
	* can be redefined in different comics modules
	*/
	function getFileName()
	{
		$match_string = "/[^\/]*$/i";
		
		if(preg_match($match_string, $this->img_url, $matches))
		{
			$this->img_name=$matches[0];
			return true;
		}
		else
		return false;
	}	
	
	/**
	* @return -1,0 or +1
	* @param $a phpgrabcomics object
	* @param $b phpgrabcomics object
	* @desc Sorts phpgrabcomics objects
 	*/
	function sort($a,$b)
	{
		return strcmp($a->name, $b->name);	
	}
	
	/**
	* @return false or the mime type
	* @param $string
	* @desc checks if the type of the string is valid	
 	*/
	function rCheckMime($string)
	{
		$string = substr($string, 0, 4);
		$tmp_return = False;
		
		while(list($key,$val) = each($this->valid_mimes))
		{
			// length of magic_number
			$tmp_length = strlen($val[1]);
			// type of array element (string, beshort, etc)
			$type = $val[2];
			$tmp_string = substr($string, 0, $tmp_length);
			
			if($val[2]=="string" and $tmp_string==$val[1])
			{
				$tmp_return = $val;				
				break;
			}
			elseif($val[2]=="beshort")
			{
				$a = ord(substr($tmp_string,0,1));
				$b = ord(substr($tmp_string,1,1));
				$c = "0x".dechex($a).dechex($b);
		
				if((int)$val[1] ==(int)$c)
				{
					$tmp_return = $val;
					break;
				}			
			}
		}	
		
		return $tmp_return;
	}
}
?>